'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _elements = require('./elements');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * defaultOptions
 * @type {{
 *     customElements: boolean,
 *     independent   : boolean
 * }}
 */
var defaultOptions = {

    /**
     * possible values: false, "v0", "v1"
     * customElements {boolean|string}
     */
    customElements: false,

    /**
     * on true : @component (nothing changed)
     * on false: @component convert to
     * _component(options){
     *     ...code...
     *     depend on customElements = v0, v1
     *     ...code...
     * }
     * independent {boolean}
     */
    independent: true,

    /**
     * Safari´s and IE´s native Element-Class it not a function.
     * We have to map them to a function otherwise it throw an error.
     *
     * if (typeof HTMLElement === 'object'){
     *     let _Element = function(){};
     *     _Element.prototype = HTMLElement.prototype;
     *     HTMLElement = _Element;
     * }
     *
     * Foo extends HTMLElement {
     *     ..code..
     * }
     *
     */
    polypill: ['elementClassToFunction']

};

/**
 * getDecorator
 * @param decoratorName
 * @returns {object|null}
 */
var getDecorator = function getDecorator(decoratorName) {

    var decorators = this.decorators || [];

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = (0, _getIterator3.default)(decorators), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var decorator = _step.value;
            var name = decorator.expression.callee.name;

            if (decoratorName === name) {
                return decorator;
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return null;
};

/**
 * getElementClassByName
 * @param name {string}
 * @param elements {object}
 * @returns {*|string}
 */
var getElementClassByName = function getElementClassByName(name, elements) {

    var element = elements[name] || 'HTMLElement';
    return element;
};

/**
 * getArguments
 * @param t {object}
 * @returns {{extends: string}}
 */
var getArguments = function getArguments(t) {

    // set default
    var args = {
        extends: 'div'
    };

    var _expression$arguments = (0, _slicedToArray3.default)(this.expression.arguments, 1),
        firstArgument = _expression$arguments[0];

    if (!t.isObjectExpression(firstArgument)) {
        return args;
    }

    var properties = firstArgument.properties;
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
        for (var _iterator2 = (0, _getIterator3.default)(properties), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var property = _step2.value;

            args[property.key.name] = property.value.value;
        }
    } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
            }
        } finally {
            if (_didIteratorError2) {
                throw _iteratorError2;
            }
        }
    }

    return args;
};

/**
 * addSuperClass
 * @param element {string}
 * @param t {object}
 */
var addSuperClass = function addSuperClass(element, t) {

    var identifier = t.identifier(element);
    this.superClass = identifier;
};

/**
 * add static string property
 * @param prop {string}
 * @param value {string}
 * @param t {object}
 */
var addStaticStringProperty = function addStaticStringProperty(prop, value, t) {

    var a = t.classProperty(t.identifier(prop), t.stringLiteral(value));
    a.static = true;
    this.body.body.push(a);
};

/**
 * addStaticGetterProperty
 * @param type {string}
 * @param superClass {string}
 * @param t {object}
 */
var addStaticGetterProperty = function addStaticGetterProperty(type, superClass, t) {

    var classMethodNode = t.classMethod('get', t.identifier(type), [], t.blockStatement([t.returnStatement(t.stringLiteral(superClass))]), false, true);

    this.body.body.push(classMethodNode);
};

/**
 * plugin
 * @astexplorer: https://astexplorer.net (useful for building plugin)
 * @param t {types}
 * @returns {{visitor: {ClassDeclaration: (function(*=))}}}
 */
var plugin = function plugin(_ref) {
    var t = _ref.types;


    return {
        visitor: {

            /**
             * ClassDeclaration
             * @param node {object}
             * @param opts {object}
             */
            ClassDeclaration: function ClassDeclaration() {
                var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : path,
                    node = _ref2.node;

                var _ref3 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : state,
                    opts = _ref3.opts;

                var options = (0, _assign2.default)({}, defaultOptions, opts);

                var component = getDecorator.call(node, 'component');
                if (!component) {

                    /**
                     * If no @component but HTML... as superClass
                     * and no extends() then add addStaticGetterProperty
                     */
                    return;
                }

                var className = node.id.name;
                addStaticStringProperty.call(node, '$$componentName', className, t);

                var superClass = getArguments.call(component, t).extends;
                var element = getElementClassByName(superClass, _elements.elements);
                addSuperClass.call(node, element, t);

                if (/div/.test(superClass)) {
                    return;
                }

                addStaticGetterProperty.call(node, 'extends', superClass, t);
            }
        }
    };
};

exports.default = plugin;